﻿using AIWolf.Lib;
using System.Collections.Generic;

namespace WhiteWolf.Arrangement
{
    /// <summary>
    /// 1日分のゲーム情報
    /// 1日は昼の会話開始～夜時間結果が出るまでとする
    /// </summary>
    public interface IDayGameInfo
    {

        /// <summary>何日目か</summary>
        int Day { get; }

        #region 昼の情報

        /// <summary>昼の生存エージェント</summary>
        IEnumerable<Agent> DayTimeAliveAgent { get; }

        /// <summary>通常会話リスト</summary>
        IEnumerable<ContentUtterance> TalkList { get; }

        #endregion


        #region 夕方の情報

        /// <summary>追放投票リスト[再投票回数]</summary>
        IReadOnlyList<VoteCollection> VoteList { get; }

        /// <summary>この日の夕方に追放したエージェント</summary>
        Agent ExecuteAgent { get; }

        #endregion


        #region 夜の情報

        /// <summary>夜の生存エージェント(当日の昼はNull)</summary>
        IEnumerable<Agent> NightAliveAgent { get; }

        /// <summary>翌朝に死体で発見されたエージェント</summary>
        IEnumerable<Agent> AttackDeadAgent { get; }

        /// <summary>占い結果(占い師以外はnull)</summary>
        AIWolf.Lib.Judge DivineResult { get; }

        /// <summary>霊能結果(霊能者以外はnull)</summary>
        AIWolf.Lib.Judge MediumResult { get; }

        /// <summary>襲撃の対象になったエージェント</summary>
        Agent TryAttackAgent { get; }

        /// <summary>護衛の対象になったエージェント</summary>
        Agent GuardAgent { get; }

        /// <summary>襲撃投票リスト[再投票回数]</summary>
        IReadOnlyList<VoteCollection> AttackVoteList { get; }

        /// <summary>囁き会話リスト</summary>
        IEnumerable<ContentUtterance> WhisperList { get; }

        #endregion

        /// <summary>情報を更新する</summary>
        void Update(IGameInfo gameInfo);

    }
}
