﻿using AIWolf.Lib;
using System.Collections.Generic;

namespace WhiteWolf.Arrangement
{
    /// <summary>
    /// 内訳(役職の組み合わせ)パターン
    /// </summary>
    public interface IPattern
    {

        /// <summary>人狼エージェントの一覧</summary>
        IEnumerable<Agent> Wolves { get; }

        /// <summary>狂人エージェントの一覧</summary>
        IEnumerable<Agent> Possessed { get; }

        /// <summary>
        /// 特定のエージェントが人狼であることを表すビットパターン
        /// 最下位ビットがAgent[01]、最上位ビットがAgent[32]に対応
        /// ビットが1であれば条件を満たす
        /// </summary>
        int IsWerewolfBits { get; }

        /// <summary>
        /// 特定のエージェントが裏切り者であることを表すビットパターン
        /// 最下位ビットがAgent[01]、最上位ビットがAgent[32]に対応
        /// ビットが1であれば条件を満たす
        /// </summary>
        int IsPossessedBits { get; }

        /// <summary>
        /// 特定のエージェントが人狼陣営であることを表すビットパターン
        /// 最下位ビットがAgent[01]、最上位ビットがAgent[32]に対応
        /// ビットが1であれば条件を満たす
        /// </summary>
        int IsWerewolfTeamBits { get; }

        /// <summary>
        /// 特定のエージェントが人狼陣営ではないことを表すビットパターン
        /// 最下位ビットがAgent[01]、最上位ビットがAgent[32]に対応
        /// ビットが1であれば条件を満たす
        /// </summary>
        int IsNotWerewolfTeamBits { get; }
    }
}
