﻿using AIWolf.Lib;
using System.Collections.Generic;
using System.Linq;

namespace WhiteWolf.Arrangement
{
    /// <summary>
    /// 新規ViewPointを生成する
    /// </summary>
    public class ViewpointFactory
    {

        /// <summary>
        /// 新規ViewPointを取得する。
        /// 現状は3狼1狂と1狼1狂のみ対応。
        /// </summary>
        /// <param name="gameSetting"></param>
        /// <returns></returns>
        public static IViewpoint GetViewPoint(IGameSetting gameSetting)
        {
            var MonsterSidePattern = new List<IPattern>();

            // ３狼１狂
            if (gameSetting.RoleNumMap[Role.WEREWOLF] == 3 && gameSetting.RoleNumMap[Role.POSSESSED] == 1)
            {
                MonsterSidePattern = Get3W1PPatternList(gameSetting);
                return new Viewpoint(MonsterSidePattern);
            }

            // １狼１狂
            if (gameSetting.RoleNumMap[Role.WEREWOLF] == 1 && gameSetting.RoleNumMap[Role.POSSESSED] == 1)
            {
                MonsterSidePattern = Get1W1PPatternList(gameSetting);
                return new Viewpoint(MonsterSidePattern);
            }

            return new Viewpoint(MonsterSidePattern);
        }

        /// <summary>
        /// 内容を指定して新規ViewPointを取得する。
        /// </summary>
        /// <param name="src"></param>
        /// <returns></returns>
        public static IViewpoint CloneViewPoint(IEnumerable<IPattern> src)
        {
            return new Viewpoint(src);
        }

        private static List<IPattern> Get3W1PPatternList(IGameSetting gameSetting)
        {
            var MonsterSidePattern = new List<IPattern>();
            int playerNum = gameSetting.PlayerNum;
            var singleAgentList = Enumerable.Range(0, playerNum + 1).Select(i => new List<Agent>() { Agent.GetAgent(i) }).ToList();

            for (int wolfAcnt = 1; wolfAcnt <= playerNum - 2; wolfAcnt++)
            {
                for (int wolfBcnt = wolfAcnt + 1; wolfBcnt <= playerNum - 1; wolfBcnt++)
                {
                    for (int wolfCcnt = wolfBcnt + 1; wolfCcnt <= playerNum; wolfCcnt++)
                    {
                        List<Agent> wolfAgent = new List<Agent>
                            { Agent.GetAgent(wolfAcnt), Agent.GetAgent(wolfBcnt), Agent.GetAgent(wolfCcnt) };

                        for (int possessedcnt = 1; possessedcnt <= playerNum; possessedcnt++)
                        {
                            if (possessedcnt != wolfAcnt && possessedcnt != wolfBcnt && possessedcnt != wolfCcnt)
                            {
                                IEnumerable<Agent> posAgent = singleAgentList[possessedcnt];

                                IPattern pattern = new Pattern(wolfAgent, posAgent);
                                MonsterSidePattern.Add(pattern);
                            }
                        }
                    }
                }
            }

            return MonsterSidePattern;
        }

        private static List<IPattern> Get1W1PPatternList(IGameSetting gameSetting)
        {
            var MonsterSidePattern = new List<IPattern>();

            for (int wolfAcnt = 1; wolfAcnt <= gameSetting.PlayerNum; wolfAcnt++)
            {
                List<Agent> wolfAgent = new List<Agent>
                    {
                        Agent.GetAgent( wolfAcnt )
                    };
                for (int possessedcnt = 1; possessedcnt <= gameSetting.PlayerNum; possessedcnt++)
                {
                    if (possessedcnt != wolfAcnt)
                    {
                        List<Agent> posAgent = new List<Agent> { Agent.GetAgent(possessedcnt) };
                        IPattern pattern = new Pattern(wolfAgent, posAgent);
                        MonsterSidePattern.Add(pattern);
                    }
                }
            }

            return MonsterSidePattern;
        }

    }
}
