﻿using AIWolf.Lib;
using System;
using System.Linq;
using WhiteWolf.ActionPlan;
using WhiteWolf.Arrangement;
using WhiteWolf.Guess;

namespace WhiteWolf.Talk
{
    public class SampleTalkGenerator : ITalkGenerator
    {
        Random rnd = new Random();

        public string GetTalkText(ITotalityGameInfo gameInfo, ITotalGuess guess, IActionPlan actionPlan)
        {
            // 自分が死んでいたら発話不可能（適当な内容を返す）
            if (!gameInfo.AliveAgentList.Contains(gameInfo.Me))
            {
                return Content.OVER.Text;
            }

            // 占い師固有の動き(他の動きより優先)
            if (gameInfo.MyRole == Role.SEER)
            {
                // 占CO
                if (!gameInfo.ComingOuts.Any(co => co.IsEnable && co.Agent.Equals(gameInfo.Me)))
                {
                    return new Content(new ComingoutContentBuilder(gameInfo.Me, gameInfo.Me, Role.SEER)).Text;
                }

                // 5人村で白引きしていたら偽黒を出す
                if (gameInfo.GameSetting.PlayerNum == 5 && gameInfo.TodayGameInfo.Day == 1 && gameInfo.TodayGameInfo.TalkList.Count() == 5 && gameInfo.YesterdayGameInfo.DivineResult.Result == Species.HUMAN)
                {
                    var seers = gameInfo.ComingOuts.Where(co => co.Role == Role.SEER).Select(co => co.Agent).ToList();
                    if (seers.Count <= 2 && !seers.Any(seer => gameInfo.YesterdayGameInfo.DivineResult.Target.Equals(seer)))
                    {
                        var target = actionPlan.DivineTarget;
                        return new Content(new DivinedResultContentBuilder(gameInfo.Me, target, Species.WEREWOLF)).Text;
                    }
                }

                // 占い結果の報告
                if (!gameInfo.SeerJudges.Any(judge => judge.Agent.Equals(gameInfo.Me) && judge.JudgeTalk.Day == gameInfo.TodayGameInfo.Day))
                {
                    var judge = gameInfo.YesterdayGameInfo?.DivineResult;
                    if (judge != null)
                    {
                        return new Content(new DivinedResultContentBuilder(gameInfo.Me, judge.Target, judge.Result)).Text;
                    }
                }
            }

            // 霊能者固有の動き(他の動きより優先)
            if (gameInfo.MyRole == Role.MEDIUM)
            {
                // 霊CO
                if (gameInfo.MyRole == Role.MEDIUM && !gameInfo.ComingOuts.Any(co => co.IsEnable && co.Agent.Equals(gameInfo.Me)))
                {
                    return new Content(new ComingoutContentBuilder(gameInfo.Me, gameInfo.Me, Role.MEDIUM)).Text;
                }
                // 霊能結果の報告
                if (gameInfo.YesterdayGameInfo?.ExecuteAgent != null && !gameInfo.MediumJudges.Any(judge => judge.Agent.Equals(gameInfo.Me) && judge.JudgeTalk.Day == gameInfo.TodayGameInfo.Day))
                {
                    var judge = gameInfo.YesterdayGameInfo.MediumResult;
                    return new Content(new IdentContentBuilder(gameInfo.Me, judge.Target, judge.Result)).Text;
                }
            }

            // 狂人固有の動き(他の動きより優先)
            if (gameInfo.MyRole == Role.POSSESSED)
            {
                // 占CO
                if (!gameInfo.ComingOuts.Any(co => co.IsEnable && co.Agent.Equals(gameInfo.Me)))
                {
                    return new Content(new ComingoutContentBuilder(gameInfo.Me, gameInfo.Me, Role.SEER)).Text;
                }
                // 占い結果の報告
                if (!gameInfo.SeerJudges.Any(judge => judge.Agent.Equals(gameInfo.Me) && judge.JudgeTalk.Day == gameInfo.TodayGameInfo.Day))
                {
                    var target = actionPlan.DivineTarget;

                    // 確率で人間か人狼か決める
                    var result = Species.HUMAN;
                    if (rnd.NextDouble() < 0.3)
                    {
                        result = Species.WEREWOLF;
                    }

                    // 狂人視点で確定白であれば黒を出す
                    if (!gameInfo.SelfRealRoleViewTrustInfo.Patterns.Any(pattern => pattern.Wolves.Contains(target)))
                    {
                        result = Species.WEREWOLF;
                    }

                    // 占い先が人狼確定、もしくは人間確定であれば破綻しない結果にする
                    var wolfParrenCount = gameInfo.SelfViewTrustInfo.Patterns.Count(pattern => pattern.Wolves.Contains(target));
                    if (wolfParrenCount == gameInfo.SelfViewTrustInfo.Patterns.Count())
                    {
                        result = Species.WEREWOLF;
                    }
                    if (wolfParrenCount == 0)
                    {
                        result = Species.HUMAN;
                    }

                    return new Content(new DivinedResultContentBuilder(gameInfo.Me, target, result)).Text;
                }
            }


            // 投票先と宣言先が合っていなければ、投票先を宣言する
            var saidVote = new VoteCollection(gameInfo.AliveAgentList, gameInfo.TodayGameInfo.TalkList);
            if (saidVote.VoteMap.ContainsKey(gameInfo.Me))
            {
                var saidTarget = saidVote.VoteMap[gameInfo.Me];
                if (!actionPlan.VoteTarget.Equals(saidTarget))
                {
                    return new Content(new VoteContentBuilder(gameInfo.Me, actionPlan.VoteTarget)).Text;
                }
            }

            return Content.OVER.Text;
        }

    }
}
