﻿using AIWolf.Lib;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System.Collections.Generic;

namespace WhiteWolf.Arrangement
{
    [TestClass]
    public class IsMatchPatternTest
    {
        Pattern pattern_123_4 = new Pattern(new List<Agent>() { Agent.GetAgent(1), Agent.GetAgent(2), Agent.GetAgent(3) }, new List<Agent>() { Agent.GetAgent(4) });

        [TestMethod]
        [DynamicData(nameof(GetDynamicDataTestData), DynamicDataSourceType.Method)]
        public void IsMatchTest(IsMatchPattern con, bool expected)
        {
            Assert.AreEqual(expected, con.IsMatch(pattern_123_4));
        }

        public static IEnumerable<object[]> GetDynamicDataTestData()
        {
            // 条件なし
            yield return new object[] {
                new IsMatchPattern(
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition),
                true
            };

            // 狼一致
            yield return new object[] {
                new IsMatchPattern(
                    new List<Agent>() { Agent.GetAgent(1), Agent.GetAgent(2), Agent.GetAgent(3) },
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition),
                true
            };

            // 狼不一致
            yield return new object[] {
                new IsMatchPattern(
                    new List<Agent>() { Agent.GetAgent(4) },
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition),
                false
            };

            // 狂人一致
            yield return new object[] {
                new IsMatchPattern(
                    IsMatchPattern.NoCondition,
                    new List<Agent>() { Agent.GetAgent(4) },
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition),
                true
            };

            // 狂人不一致
            yield return new object[] {
                new IsMatchPattern(
                    IsMatchPattern.NoCondition,
                    new List<Agent>() { Agent.GetAgent(5) },
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition),
                false
            };

            // 狼陣営一致
            yield return new object[] {
                new IsMatchPattern(
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition,
                    new List<Agent>() { Agent.GetAgent(1), Agent.GetAgent(2), Agent.GetAgent(3), Agent.GetAgent(4) },
                    IsMatchPattern.NoCondition),
                true
            };

            // 狼陣営不一致
            yield return new object[] {
                new IsMatchPattern(
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition,
                    new List<Agent>() { Agent.GetAgent(5) },
                    IsMatchPattern.NoCondition),
                false
            };

            // 村陣営一致
            yield return new object[] {
                new IsMatchPattern(
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition,
                    new List<Agent>() { Agent.GetAgent(5), Agent.GetAgent(15), Agent.GetAgent(32) }),
                true
            };

            // 村陣営不一致
            yield return new object[] {
                new IsMatchPattern(
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition,
                    IsMatchPattern.NoCondition,
                    new List<Agent>() { Agent.GetAgent(4) }),
                false
            };
        }

    }
}
