﻿using AIWolf.Lib;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Moq;
using System.Collections.Generic;
using System.Linq;
using WhiteWolf.Arrangement;

namespace WhiteWolfTest.Arrangement
{
    [TestClass]
    public class ViewpointTest
    {

        [TestMethod]
        public void RemovePatternsTest()
        {
            // Mockを作成
            Mock<IGameSetting> gameSetting = new Mock<IGameSetting>();
            gameSetting.Setup(p => p.PlayerNum).Returns(15);
            gameSetting.Setup(p => p.RoleNumMap).Returns(new Dictionary<Role, int>() { { Role.WEREWOLF, 3 }, { Role.POSSESSED, 1 } });

            // 視点を作成
            IViewpoint vp = ViewpointFactory.GetViewPoint(gameSetting.Object);

            // 初期の組み合わせ数
            Assert.AreEqual(5460, vp.Patterns.Count());

            // 1人が人外のパターンを除外した組み合わせ数
            vp.RemovePatterns(p => p.Wolves.Contains(Agent.GetAgent(1)) || p.Possessed.Contains(Agent.GetAgent(1)));
            Assert.AreEqual(4004, vp.Patterns.Count());
        }

        [TestMethod]
        public void InclusionViewpointsTest()
        {
            // Mockを作成
            Mock<IGameSetting> gameSetting = new Mock<IGameSetting>();
            gameSetting.Setup(p => p.PlayerNum).Returns(15);
            gameSetting.Setup(p => p.RoleNumMap).Returns(new Dictionary<Role, int>() { { Role.WEREWOLF, 3 }, { Role.POSSESSED, 1 } });

            // 視点を作成
            IViewpoint parentVp = ViewpointFactory.GetViewPoint(gameSetting.Object);
            IViewpoint childVp = parentVp.Clone();
            parentVp.AddInclusionViewpoint(childVp);

            // 初期の組み合わせ数
            Assert.AreEqual(5460, parentVp.Patterns.Count());
            Assert.AreEqual(5460, childVp.Patterns.Count());

            // 1人が人外のパターンを除外した組み合わせ数
            parentVp.RemovePatterns(p => p.Wolves.Contains(Agent.GetAgent(1)) || p.Possessed.Contains(Agent.GetAgent(1)));
            Assert.AreEqual(4004, parentVp.Patterns.Count());
            Assert.AreEqual(4004, childVp.Patterns.Count());
        }



    }
}
